package model

import (
	model2 "cold-logistics/common/model"
	"errors"
	"gorm.io/gorm"
)

type SysDept struct {
	model2.Model
	ParentId int    `json:"parentId" gorm:""`                        // 上级部门
	Path     string `json:"-" gorm:"size:255;" swaggerignore:"true"` // 路径
	Name     string `json:"name"  gorm:"size:128;"`                  // 部门名称
	Sort     int    `json:"-" gorm:"size:4;"`                        // 排序
	Status   int    `json:"-" gorm:"size:4;"`                        // 状态 1-停用 2-正常
	Remark   string `json:"remark"  gorm:"size:1024;"`               // 备注
	model2.ControlBy
	model2.ModelTime
}

func (SysDept) TableName() string {
	return "sys_dept"
}

type SysDeptOmit struct {
	Id   int    `json:"id,omitempty"`
	Path string `json:"path,omitempty"` // 路径
	Name string `json:"name,omitempty"` // 部门名称
}

func (SysDeptOmit) TableName() string {
	return "sys_dept"
}

func (e *SysDept) Generate() model2.ActiveRecord {
	o := *e
	return &o
}

func (e *SysDept) GetId() interface{} {
	return e.Id
}

func (e *SysDept) BeforeDelete(_ *gorm.DB) (err error) {
	if e.Id == 1 {
		return errors.New("禁止删除系统部门")
	}
	return
}
